import React from "react";
import Animated from "react-native-reanimated";
import { SpringDriver } from "@tleef/react-native-reanimated-utils/lib";
import withStage from "./withStage";
import SceneTransition from "./SceneTransition";
import TransitionReceipt from "./TransitionReceipt";
const { Value, SpringUtils, set, cond, block, eq, not, clockRunning, call } = Animated;
export class SceneTransitionerClass extends React.Component {
    constructor(props) {
        super(props);
        this.transition = ({ from, to, springConfig }) => {
            const { stage } = this.props;
            const fromScene = stage.getScene(from);
            const toScene = stage.getScene(to);
            if (!fromScene) {
                stage.onAddScene(from, this._handleAddScene);
            }
            if (!toScene) {
                stage.onAddScene(to, this._handleAddScene);
            }
            if (from !== this.state.from || to !== this.state.to) {
                this._resetAnimation(springConfig);
            }
            this._inOut.setValue(1);
            this.setState({
                from,
                fromScene,
                to,
                toScene
            });
            return new TransitionReceipt(this._inOut, this._enterStartListeners, this._enterEndListeners, this._leaveStartListeners, this._leaveEndListeners);
        };
        this.reset = () => {
            this._resetAnimation();
            this._resetState();
        };
        this._resetAnimation = (springConfig = {}) => {
            const defaultSpringConfig = SpringUtils.makeDefaultConfig();
            this._driver = new SpringDriver({
                stiffness: springConfig.stiffness || defaultSpringConfig.stiffness,
                mass: springConfig.mass || defaultSpringConfig.mass,
                damping: springConfig.damping || defaultSpringConfig.damping,
                overshootClamping: springConfig.overshootClamping || defaultSpringConfig.overshootClamping,
                restDisplacementThreshold: springConfig.restDisplacementThreshold ||
                    defaultSpringConfig.restDisplacementThreshold,
                restSpeedThreshold: springConfig.restSpeedThreshold ||
                    defaultSpringConfig.restSpeedThreshold
            });
            this._inOut = new Value(1);
            this._prevInOut = new Value(0);
            // ENTER
            const handleLeaveCancelled = cond(eq(this._prevInOut, -1), call([], () => this._onLeaveEnd(true)));
            const handleEnterStart = cond(not(eq(this._prevInOut, 1)), call([], this._onEnterStart));
            const handleEnterEnd = cond(not(clockRunning(this._driver.clock)), call([], () => this._onEnterEnd(false)));
            const handleEnter = block([
                handleLeaveCancelled,
                handleEnterStart,
                this._driver.run(),
                handleEnterEnd,
                set(this._prevInOut, 1)
            ]);
            // LEAVE
            const handleEnterCancelled = cond(eq(this._prevInOut, 1), call([], () => this._onEnterEnd(true)));
            const handleLeaveStart = cond(not(eq(this._prevInOut, -1)), call([], this._onLeaveStart));
            const handleLeaveEnd = cond(not(clockRunning(this._driver.clock)), call([], () => this._onLeaveEnd(false)));
            const handleLeave = block([
                handleEnterCancelled,
                handleLeaveStart,
                this._driver.rev(),
                handleLeaveEnd,
                set(this._prevInOut, -1)
            ]);
            this._drive = cond(eq(this._inOut, 1), handleEnter, cond(eq(this._inOut, -1), handleLeave));
            this._enterStartListeners = [];
            this._enterEndListeners = [];
            this._leaveStartListeners = [];
            this._leaveEndListeners = [];
        };
        this._resetState = () => {
            this.setState({
                from: undefined,
                fromScene: undefined,
                to: undefined,
                toScene: undefined
            });
        };
        this._onEnterStart = () => {
            this._enterStartListeners.forEach(fn => fn());
        };
        this._onEnterEnd = (cancelled) => {
            this._enterEndListeners.forEach(fn => fn(cancelled));
            if (!cancelled) {
                this.reset();
            }
        };
        this._onLeaveStart = () => {
            this._leaveStartListeners.forEach(fn => fn());
        };
        this._onLeaveEnd = (cancelled) => {
            this._leaveEndListeners.forEach(fn => fn(cancelled));
            if (!cancelled) {
                this.reset();
            }
        };
        this._handleAddScene = (sceneId, scene) => {
            const { from, to } = this.state;
            if (from === sceneId) {
                this.setState({
                    fromScene: scene
                });
            }
            else if (to === sceneId) {
                this.setState({
                    toScene: scene
                });
            }
            this.props.stage.removeAddSceneListener(sceneId, this._handleAddScene);
        };
        this.state = {};
        this._resetAnimation();
    }
    render() {
        const { fromScene, toScene } = this.state;
        if (!fromScene || !toScene) {
            return null;
        }
        return (<React.Fragment>
        <Animated.Code>{() => this._drive}</Animated.Code>
        <SceneTransition from={fromScene} to={toScene} position={this._driver.value}/>
      </React.Fragment>);
    }
}
export default withStage(SceneTransitionerClass);
//# sourceMappingURL=SceneTransitioner.js.map